@@ -551,7 +551,7 @@ class Command(object):
         return {'powerstate': self._get_power_state(
             bridge_request=bridge_request)}
 
-    def set_identify(self, on=True, duration=None):
+    def set_identify(self, on=True, duration=None, blink=False):
         """Request identify light
 
         Request the identify light to turn off, on for a duration,
@@ -563,10 +563,14 @@ class Command(object):
         """
         self.oem_init()
         try:
-            self._oem.set_identify(on, duration)
+            self._oem.set_identify(on, duration, blink)
+            return
+        except exc.BypassGenericBehavior:
             return
         except exc.UnsupportedFunctionality:
             pass
+        if blink:
+            raise exc.IpmiException('Blink not supported with generic IPMI')
         if duration is not None:
             duration = int(duration)
             if duration > 255:
@@ -650,6 +654,14 @@ class Command(object):
         return sel.EventHandler(self.init_sdr(), self).decode_pet(specifictrap,
                                                                   petdata)
 
+    def get_ikvm_methods(self):
+        self.oem_init()
+        return self._oem.get_ikvm_methods()
+
+    def get_ikvm_launchdata(self):
+        self.oem_init()
+        return self._oem.get_ikvm_launchdata()
+
     def get_inventory_descriptions(self):
         """Retrieve list of things that could be inventoried
 
@@ -778,6 +790,18 @@ class Command(object):
             summary['badreadings'] = fallbackreadings
         return summary
 
+    def get_system_power_watts(self):
+        self.oem_init()
+        return self._oem.get_system_power_watts(self)
+    
+    def get_inlet_temperature(self):
+        self.oem_init()
+        return self._oem.get_inlet_temperature(self)
+
+    def get_average_processor_temperature(self):
+        self.oem_init()
+        return self._oem.get_average_processor_temperature(self)
+
     def get_sensor_reading(self, sensorname):
         """Get a sensor reading by name
 
@@ -818,6 +842,12 @@ class Command(object):
         fetchdata = fetched['data']
         if bytearray(fetchdata)[0] != 17:
             return None
+        if param == 0x14:
+            vlaninfo = struct.unpack('<H', fetchdata[1:])[0]
+            if vlaninfo & 0x8000 == 0x8000:
+                return vlaninfo & 0xfff
+            else:
+                return 'off'
         if len(fetchdata) == 5:  # IPv4 address
             if prefixlen:
                 return _mask_to_cidr(fetchdata[1:])
@@ -837,9 +867,9 @@ class Command(object):
         else:
             raise Exception("Unrecognized data format " + repr(fetchdata))
 
-    def get_extended_bmc_configuration(self):
+    def get_extended_bmc_configuration(self, hideadvanced=True):
         self.oem_init()
-        return self._oem.get_extended_bmc_configuration()
+        return self._oem.get_extended_bmc_configuration(hideadvanced=hideadvanced)
 
     def get_bmc_configuration(self):
         self.oem_init()
@@ -870,7 +900,7 @@ class Command(object):
         self._oem.clear_system_configuration()
 
     def set_net6_configuration(self, static_addresses=None, static_gateway=None, channel=None):
-        if static_addrs is None and static_gateway is None:
+        if static_addresses is None and static_gateway is None:
             return
         if channel is None:
             channel = self.get_network_channel()
@@ -907,12 +937,12 @@ class Command(object):
         ip6gd = bytearray(ip6g['data'])
         if ip6gd[0] != 0x11:
             raise Exception('Unsupported reply')
-        gwa = socket.inet_ntop(socket.AF_INET6, ip6gd[1:17])
+        gwa = socket.inet_ntop(socket.AF_INET6, bytes(ip6gd[1:17]))
         retdata['static_gateway'] = gwa
         return retdata
 
     def set_net_configuration(self, ipv4_address=None, ipv4_configuration=None,
-                              ipv4_gateway=None, channel=None):
+                              ipv4_gateway=None, vlan_id=None, channel=None):
         """Set network configuration data.
 
         Apply desired network configuration data, leaving unspecified
@@ -926,7 +956,7 @@ class Command(object):
         :param channel:  LAN channel to configure, defaults to autodetect
         """
         if (ipv4_address is None and ipv4_configuration is None
-                and ipv4_gateway is None):
+                and ipv4_gateway is None and vlan_id is None):
             return
         if channel is None:
             channel = self.get_network_channel()
@@ -953,6 +983,12 @@ class Command(object):
         if ipv4_gateway is not None:
             cmddata = bytearray((channel, 12)) + socket.inet_aton(ipv4_gateway)
             self.xraw_command(netfn=0xc, command=1, data=cmddata)
+        if vlan_id in ('off', 0, '0'):
+            cmddata = bytearray((channel, 0x14, 0, 0))
+            self.xraw_command(netfn=0xc, command=1, data=cmddata)
+        elif vlan_id is not None:
+            cmddata = bytearray((channel, 0x14)) + struct.pack('<H', int(vlan_id) | 0x8000)
+            self.xraw_command(netfn=0xc, command=1, data=cmddata)
 
     def get_storage_configuration(self):
         """"Get storage configuration data
@@ -1047,6 +1083,7 @@ class Command(object):
             retdata['ipv4_gateway_mac'] = self._fetch_lancfg_param(channel, 13)
             retdata['ipv4_backup_gateway_mac'] = self._fetch_lancfg_param(
                 channel, 15)
+        retdata['vlan_id'] = self._fetch_lancfg_param(channel, 0x14)
         self.oem_init()
         self._oem.add_extra_net_configuration(retdata, channel)
         return retdata
@@ -1636,6 +1673,10 @@ class Command(object):
         r['privilege_level'] = privilege_levels[data[1] & 0b00001111]
         return r
 
+    def get_screenshot(self, outfile):
+        self.oem_init()
+        return self._oem.get_screenshot(outfile)
+
     def get_channel_info(self, channel=None):
         """Get channel info
 
@@ -2138,14 +2179,14 @@ class Command(object):
 
         return True
 
-    def get_firmware(self, components=()):
+    def get_firmware(self, components=(), category=None):
         """Retrieve OEM Firmware information"""
 
         self.oem_init()
         mcinfo = self.xraw_command(netfn=6, command=1)
         major, minor = struct.unpack('BB', mcinfo['data'][2:4])
         bmcver = '{0}.{1}'.format(major, hex(minor)[2:])
-        return self._oem.get_oem_firmware(bmcver, components)
+        return self._oem.get_oem_firmware(bmcver, components, category)
 
     def get_capping_enabled(self):
         """Get PSU based power capping status
@@ -2196,6 +2237,10 @@ class Command(object):
         self.oem_init()
         return self._oem.get_graphical_console()
 
+    def get_update_status(self):
+        self.oem_init()
+        return self._oem.get_update_status()
+
     def update_firmware(self, filename, data=None, progress=None, bank=None):
         """Send file to BMC to perform firmware update
 
