@@ -13,7 +13,9 @@
 # limitations under the License.
 
 import pyghmi.exceptions as exc
-
+import pyghmi.ipmi.private.constants as event_const
+import pyghmi.ipmi.sdr as ipmisdr
+import struct
 
 class OEMHandler(object):
     """Handler class for OEM capabilities.
@@ -40,6 +42,96 @@ class OEMHandler(object):
         """
         return {}
 
+    def get_screenshot(self, outfile):
+        return {}
+
+    def get_system_power_watts(self, ipmicmd):
+        # Use DCMI getpower reading command
+        rsp = ipmicmd.xraw_command(netfn=0x2c, command=2, data=(0xdc, 1, 0, 0))
+        wattage = struct.unpack('<H', rsp['data'][1:3])[0]
+        return wattage
+
+    def get_ikvm_methods(self):
+        return []
+
+    def get_ikvm_launchdata(self):
+        # no standard ikvm behavior, must be oem defined
+        return {}
+
+    def get_average_processor_temperature(self, ipmicmd):
+        # DCMI suggests preferrence for 0x37 ('Air inlet')
+        # If not that, then 0x40 ('Air inlet')
+        # in practice, some implementations use 0x27 ('External environment')
+        if not hasattr(self, '_processor_names'):
+            self._processor_names = []
+        readings = []
+        if not self._processor_names:
+            sdr = ipmicmd.init_sdr()
+            for sensename in sdr.sensors:
+                sensor = sdr.sensors[sensename]
+                if sensor.reading_type != 1:
+                    continue
+                if not sensor.baseunit:
+                    continue
+                if sensor.sensor_type != 'Temperature':
+                    continue
+                if sensor.entity == 'Processor':
+                    self._processor_names.append(sensor.sensor_name)
+        readingvalues = []
+        for procsensor in self._processor_names:
+            try:
+                reading = ipmicmd.get_sensor_reading(procsensor)
+            except exc.IpmiException:
+                continue
+            if reading.value is not None:
+                readingvalues.append(float(reading.value))
+        tmplreading = ipmisdr.SensorReading({'name': 'Average Processor Temperature', 'type': 'Temperature'}, '°C')
+        if readingvalues:
+            tmplreading.value = sum(readingvalues) / len(readingvalues)
+        else:
+            tmplreading.value = None
+            tmplreading.unavailable = 1
+        return tmplreading
+
+
+    def get_inlet_temperature(self, ipmicmd):
+        # DCMI suggests preferrence for 0x37 ('Air inlet')
+        # If not that, then 0x40 ('Air inlet')
+        # in practice, some implementations use 0x27 ('External environment')
+        if not hasattr(self, '_inlet_name'):
+            self._inlet_name = None
+        if self._inlet_name:
+            return ipmicmd.get_sensor_reading(self._inlet_name)
+        sdr = ipmicmd.init_sdr()
+        extenv = []
+        airinlets = []
+        for sensename in sdr.sensors:
+            sensor = sdr.sensors[sensename]
+            if sensor.reading_type != 1:
+                continue
+            if not sensor.baseunit:
+                continue
+            if sensor.sensor_type != 'Temperature':
+                continue
+            if sensor.entity == 'External environment':
+                if 'exhaust' in sensor.sensor_name.lower():
+                    continue
+                extenv.append(sensor.sensor_name)
+            if sensor.entity == 'Air inlet':
+                airinlets.append(sensor.sensor_name)
+        if airinlets:
+            if len(airinlets) > 1:
+                raise Exception('TODO: how to deal with multiple inlets')
+            self._inlet_name = airinlets[0]
+        elif extenv:
+            if len(extenv) > 1:
+                raise Exception('TODO: how to deal with multiple external environments')
+            self._inlet_name = extenv[0]
+        if not self._inlet_name:
+            raise exc.UnsupportedFunctionality(
+                'Unable to detect inlet sensor name for this platform')
+        return ipmicmd.get_sensor_reading(self._inlet_name)
+
     def process_event(self, event, ipmicmd, seldata):
         """Modify an event according with OEM understanding.
 
@@ -187,7 +279,7 @@ class OEMHandler(object):
         fru['oem_parser'] = None
         return fru
 
-    def get_oem_firmware(self, bmcver, components):
+    def get_oem_firmware(self, bmcver, components, category):
         """Get Firmware information."""
 
         # Here the bmc version is passed into the OEM handler, to allow
@@ -250,19 +342,23 @@ class OEMHandler(object):
         raise exc.UnsupportedFunctionality(
             'Remote storage configuration not supported on this platform')
 
+    def get_update_status(self):
+        raise exc.UnsupportedFunctionality(
+            'Firmwore update not supported on this platform')
+
     def update_firmware(self, filename, data=None, progress=None, bank=None):
         raise exc.UnsupportedFunctionality(
             'Firmware update not supported on this platform')
 
     def reseat_bay(self, bay):
         raise exc.UnsupportedFunctionality(
-            'Bay reseat not supported on this platform')
+            'Reseat not supported on this platform')
 
     def get_graphical_console(self):
         """Get graphical console launcher"""
         return ()
 
-    def add_extra_net_configuration(self, netdata):
+    def add_extra_net_configuration(self, netdata, channel=None):
         """Add additional network configuration data
 
         Given a standard netdata struct, add details as relevant from
@@ -271,19 +367,33 @@ class OEMHandler(object):
         """
         return
 
+    def get_oem_identifier(self):
+        """Get host name
+
+        """
+        return None
+
+    def set_oem_identifier(self, name):
+        """Set host name
+
+        :param name: host name to be set
+        """
+        return False
+
     def detach_remote_media(self):
         raise exc.UnsupportedFunctionality()
 
     def attach_remote_media(self, imagename, username, password):
         raise exc.UnsupportedFunctionality()
 
-    def upload_media(self, filename, progress):
-        raise exc.UnsupportedFunctionality()
+    def upload_media(self, filename, progress, data):
+        raise exc.UnsupportedFunctionality(
+            'Remote media upload not supported on this system')
 
     def list_media(self):
         raise exc.UnsupportedFunctionality()
 
-    def set_identify(self, on, duration):
+    def set_identify(self, on, duration, blink):
         """Provide an OEM override for set_identify
 
         Some systems may require an override for set identify.
@@ -314,6 +424,8 @@ class OEMHandler(object):
 
     def set_user_access(self, uid, channel, callback, link_auth, ipmi_msg,
                         privilege_level):
+        if privilege_level.startswith('custom.'):
+            raise exc.UnsupportedFunctionality()
         return  # Nothing to do
 
     def set_alert_ipv6_destination(self, ip, destination, channel):
@@ -383,3 +495,44 @@ class OEMHandler(object):
 
     def get_user_expiration(self, uid):
         return None
+    
+    def get_user_privilege_level(self, uid):
+        return None
+
+    def set_oem_extended_privilleges(self, uid):
+        """Set user extended privillege as 'KVM & VMedia Allowed'
+
+        |KVM & VMedia Not Allowed	0x00 0x00 0x00 0x00
+        |KVM Only Allowed	0x01 0x00 0x00 0x00
+        |VMedia Only Allowed	0x02  0x00 0x00 0x00
+        |KVM & VMedia Allowed	0x03 0x00 0x00 0x00
+
+        :param uid: User ID.
+        """
+        return False
+
+    def process_zero_fru(self, zerofru):
+        return self.process_fru(zerofru)
+
+    def is_valid(self, name):
+        return name is not None
+
+    def process_password(self, password, data):
+        return data
+
+    def set_server_capping(self, value):
+        """Set power capping for server
+
+        :param value: power capping value to set.
+        """
+        pass
+
+    def get_server_capping(self):
+        """Get power capping for server
+
+        :return: power capping value.
+        """
+        return None
+
+    def get_oem_event_const(self):
+        return event_const
