@@ -403,6 +403,7 @@ class SMMClient(object):
         'password_login_failures': 'passwordFailAllowdNum',
         'password_min_length': 'passwordMinLength',
         'password_lockout_period': 'passwordLockoutTimePeriod',
+        'timezone': 'timeZone',
     }
 
     fanmodes = {
@@ -414,7 +415,7 @@ class SMMClient(object):
         5: 'Boosted',
     }
 
-    def get_bmc_configuration(self):
+    def get_bmc_configuration(self, variant):
         settings = {}
         wc = self.wc
         wc.request(
@@ -422,14 +423,18 @@ class SMMClient(object):
             ('get=passwordMinLength,passwordForceChange,passwordDurationDays,'
              'passwordExpireWarningDays,passwordChangeInterval,'
              'passwordReuseCheckNum,passwordFailAllowdNum,'
-             'passwordLockoutTimePeriod'))
+             'passwordLockoutTimePeriod,timeZone'))
         rsp = wc.getresponse()
         rspbody = rsp.read()
         accountinfo = fromstring(rspbody)
         for rule in self.rulemap:
             ruleinfo = accountinfo.find(self.rulemap[rule])
             if ruleinfo is not None:
-                settings[rule] = {'value': int(ruleinfo.text)}
+                try:
+                    val = int(ruleinfo.text)
+                except ValueError:
+                    val = ruleinfo.text
+                settings[rule] = {'value': val}
         dwc = self.ipmicmd.xraw_command(0x32, 0x94)
         dwc = bytearray(dwc['data'])
         if len(dwc) not in (3, 4) or dwc[0] == 1:
@@ -448,7 +453,10 @@ class SMMClient(object):
         powercfg = self.ipmicmd.xraw_command(0x32, 0xa2)
         powercfg = bytearray(powercfg['data'])
         if len(powercfg) == 5:
-            powercfg = powercfg[1:]
+            if variant and variant >> 5:
+                powercfg = powercfg[-2:]
+            else:
+                powercfg = powercfg[1:]
         val = powercfg[0]
         if val == 2:
             val = 'N+N'
@@ -519,8 +527,11 @@ class SMMClient(object):
             }
             for baynum in range(numbays):
                 baynum += 1
-                baycapinfo = self.ipmicmd.xraw_command(
-                    0x32, 0x9d, data=[baynum])
+                try:
+                    baycapinfo = self.ipmicmd.xraw_command(
+                        0x32, 0x9d, data=[baynum])
+                except Exception:
+                    continue
                 capmin, capmax, protcap, usercap, thermcap = struct.unpack(
                     '<HHHHH', baycapinfo['data'][retoffset:retoffset + 10])
                 settings['bay{0}_user_cap'.format(baynum)] = {
@@ -535,8 +546,11 @@ class SMMClient(object):
                     'help': 'Show the current protective cap for the system '
                             'in bay {0}'.format(baynum)
                 }
-                baycapstate = self.ipmicmd.xraw_command(
-                    0x32, 0xa0, data=[baynum])
+                try:
+                    baycapstate = self.ipmicmd.xraw_command(
+                        0x32, 0xa0, data=[baynum])
+                except Exception:
+                    continue
                 baycapstate = bool(baycapstate['data'][retoffset])
                 settings['bay{0}_user_cap_active'.format(baynum)] = {
                     'value': 'Enable' if baycapstate else 'Disable',
@@ -695,6 +709,8 @@ class SMMClient(object):
                 if None in powercfg:
                     currcfg = self.ipmicmd.xraw_command(0x32, 0xa2)
                     currcfg = bytearray(currcfg['data'])
+                    if variant and variant >> 5 and len(currcfg) == 5:
+                        currcfg = currcfg[-2:]
                     if powercfg[0] is None:
                         powercfg[0] = currcfg[0]
                     if powercfg[1] is None:
@@ -746,7 +762,13 @@ class SMMClient(object):
                 rsp = {'data': [1]}
             rsp['data'] = bytearray(rsp['data'])
             if rsp['data'][0] == 2:  # shared io
-                rsp = self.ipmicmd.xraw_command(0x32, 0xa7, data=[bay - 1])
+                try:
+                    rsp = self.ipmicmd.xraw_command(0x32, 0xa7, data=[bay - 1])
+                except Exception:
+                    raise Exception('Shared IO detected trying to reseat {}, '
+                                    'but unable to determine status of '
+                                    'partner bay {}'.format(
+                                        bay, bay - 1))
                 rsp['data'] = bytearray(rsp['data'])
                 if rsp['data'][1] == 0x80:
                     raise Exception('Unable to reseat bay {0} due to bay {1} '
@@ -1018,11 +1040,18 @@ class SMMClient(object):
         rsp = wc.getresponse()
         rsp.read()
         complete = False
+        tries = 0
         while not complete:
             ipmisession.Session.pause(3)
             wc.request('POST', '/data', 'get=fwProgress,fwUpdate')
-            rsp = wc.getresponse()
-            progdata = rsp.read()
+            try:
+                rsp = wc.getresponse()
+                progdata = rsp.read()
+            except Exception:
+                if tries > 2:
+                     break
+                tries += 1
+                continue
             if rsp.status != 200:
                 raise Exception('Error applying firmware')
             progdata = fromstring(progdata)
